/*
SHL32 -- Run commands piped in from stdin, multiple commands
	from the command line

    Copyright (C) 1988-2003  Norm London, London Computing

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	norm_london@londoncomputing.com

    Compile with Visual Studio 6 or later. 
    cl shl32.c 

*/


#include <stdio.h>
#include <direct.h>
#include <ctype.h>
#include <process.h>
#include <string.h>
#include <stdlib.h>

char cmdline[1024],*c,*s,temp[20],path[_MAX_PATH];
int i,quiet=0;

void put_prompt(void) {
	if (quiet)
		return;
	fputs("(sh) ",stderr);
	fputs(getcwd(path,sizeof(path)),stderr);
	fputs("> ",stderr);
}


void main(int argc, char *argv[]) {

	if (argc > 1) {
		if (argv[1][0] == '?' || strcmp(argv[1],"-?")==0) {
			fputs("SHL32 Copyright (c) LONDON Computing, 1988, 1990, 1997, 2003\n",stderr);
			fprintf(stderr,"This program is free software; you can redistribute it and/or modify\n");
			fprintf(stderr,"it under the terms of the GNU General Public License as published by\n");
			fprintf(stderr,"the Free Software Foundation; either version 2 of the License, or\n");
			fprintf(stderr,"(at your option) any later version.\n\n");
			fprintf(stderr,"This program is distributed in the hope that it will be useful,\n");
			fprintf(stderr,"but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
			fprintf(stderr,"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
			fprintf(stderr,"GNU General Public License for more details.\n\n");
			fprintf(stderr,"You should have received a copy of the GNU General Public License\n");
			fprintf(stderr,"along with this program; if not, write to the Free Software\n");
			fprintf(stderr,"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n");

			fputs("Usage is: shl [-q] {<cmdfile | cmd1 ; cmd2 ; cmd3 ...}\n",stderr);
			fputs("shl executes multiple commands on a single line, separated by ; -OR-\n",stderr);
			fputs("it will execute commands redirected from the standard input device (console).\n",stderr);
			fputs("-q -- shell runs in quiet mode, does not prompt or echo commands.\n",stderr);
			fputs("On the command line the following substitions will be made:\n",stderr);
			fputs("%s--semicolon,    %g--greater than,     %l--less than, %b--vertical bar\n",stderr);
			fputs("%q--double quote, % followed by any other character becomes that character.\n",stderr);
			exit(0);
		}
	}

	if (argv[1][0] == '-' && (argv[1][1]=='Q' || argv[1][1] == 'q')) {
		quiet = 1;
		i = 2;
	}

	else
		i = 1;

	c=cmdline;
	put_prompt();
	if (argc > i) {
		strcpy(cmdline,argv[i]);
		for (i++; i < argc; ++i) {
			strcat(cmdline," ");
			strcat(cmdline,argv[i]);
		}
		while ((s = strchr(c,'%')) != NULL) {
			++s;
			switch(toupper(*s)) {
			case 'B':
				*(s-1) = '|';
				break;
			case 'G':
				*(s-1) = '>';
				break;
			case 'L':
				*(s-1) = '<';
				break;
			case 'Q':
				*(s-1) = '\"';
				break;
			case 'S':
				*(s-1) = ';';
				break;
			default:
				*(s-1) = *s;
				break;
			}
			*s = 0;
			c = s;
			strcat(cmdline,s+1);
		}

		c = cmdline;
		while ((s = strchr(c,';')) != NULL) {
			*s = 0;
			if (!quiet) {
				fputs(c,stderr);
				putc('\n',stderr);
			}
			strncpy(temp,c,4);
			temp[4] = 0;
			if (strcmp(strupr(temp),"EXIT") == 0)
				exit(0);
			if (temp[2]==0 && temp[1]==':')
				_chdrive(temp[0]-'@');
			else if (strncmp(temp,"CD ",3) == 0)
				_chdir(c+3);
			else if (strncmp(temp,"CHDI",4) == 0 &&
			    toupper(c[4]) == 'R' && c[5]==' ')
				_chdir(c+6);
			else
				system(c);
			c = s+1;
			put_prompt();
		}
		if (!quiet) {
			fputs(c,stderr);
			putc('\n',stderr);
		}
		strncpy(temp,c,3);
		temp[3]=0;
		strupr(temp);
		if (temp[2]==0 && temp[1]==':')
			_chdrive(temp[0]-'@');
		else
			system(c);
	}
	else {
		while (gets(cmdline) != NULL) {
			if (!quiet) {
				fputs(cmdline,stderr);
				putc('\n',stderr);
			}
			if (strcmp(strupr(cmdline),"EXIT") == 0)
				exit(0);
			if (strlen(cmdline) < sizeof(temp))
				strcpy(temp,cmdline);
			else {
				strncpy(temp,cmdline,sizeof(temp)-1);
				temp[sizeof(temp) -1] = 0;
			}
			if (temp[2]==0 && temp[1]==':')
				_chdrive(temp[0]-'@');
			else if (strncmp(temp,"CD ",3) == 0)
				_chdir(c+3);
			else if (strncmp(temp,"CHDI",4) == 0 &&
			    toupper(c[4]) == 'R' && c[5]==' ')
				_chdir(c+6);
			else
				system(cmdline);
			put_prompt();
		}
	}
}
